/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/

#include "assert.h"

#include "compare.h"
#include "stringmanip.h"
#include "dbase.h"
#include "dbaseexception.h"
#include "domoglobals.h"
#include "writertxt.h"
#include "fileio.h"
#include "qobject.h"
#include "ioexception.h"

using std::cout;
using DbaseSpace::Dbase;
using std::pair;
using std::vector;
using CompareSpace::Compare;
using TrackSpace::Track;
using FileSpace::File;
using WriterSpace::WriterTxt;
//-----------------------------------------------------------------------------------------------------------
Compare::Compare(RcMan* resourceManager, Log* logger) {
    assert(resourceManager != 0);
    fLenTolerance = 0;
    fRequireEqualCom = false;
    fRcMan = resourceManager;
    fLogger = logger;
}
//-----------------------------------------------------------------------------------------------------------
Compare::~Compare(){
    clear();
}
//-----------------------------------------------------------------------------------------------------------
void Compare::setRequireEqualCom(const bool b) {
	fRequireEqualCom = b;
}
//-----------------------------------------------------------------------------------------------------------
bool Compare::getRequireEqualCom() const {
	return fRequireEqualCom;
}
//-----------------------------------------------------------------------------------------------------------
int Compare::getLenTolerance() const {
	return fLenTolerance;
}
//-----------------------------------------------------------------------------------------------------------
void Compare::setLenTolerance(const int i) {
	fLenTolerance = i;
}
//-----------------------------------------------------------------------------------------------------------
Compare::ConstMaybeListIterator Compare::getConstMaybeBegin(){
	return fMaybeList.begin();
}
//-----------------------------------------------------------------------------------------------------------
Compare::ConstMaybeListIterator Compare::getConstMaybeEnd(){
	return fMaybeList.end();
}
//-----------------------------------------------------------------------------------------------------------
Compare::ConstYesListIterator Compare::getConstYesBegin(){
	return fYesList.begin();
}
//-----------------------------------------------------------------------------------------------------------
Compare::ConstYesListIterator Compare::getConstYesEnd(){
	return fYesList.end();
}
//-----------------------------------------------------------------------------------------------------------
Compare::ConstDiffVerListIterator Compare::getConstDiffVerBegin(){
	return fDiffVerList.begin();
}
//-----------------------------------------------------------------------------------------------------------
Compare::ConstDiffVerListIterator Compare::getConstDiffVerEnd(){
	return fDiffVerList.end();
}
//-----------------------------------------------------------------------------------------------------------
File::ConstFileIterator Compare::getConstNoBegin(){
	return fNoList.begin();
}
//-----------------------------------------------------------------------------------------------------------
File::ConstFileIterator Compare::getConstNoEnd(){
	return fNoList.end();
}//-----------------------------------------------------------------------------------------------------------
Compare::MaybeListIterator Compare::getMaybeBegin(){
	return fMaybeList.begin();
}
//-----------------------------------------------------------------------------------------------------------
Compare::MaybeListIterator Compare::getMaybeEnd(){
	return fMaybeList.end();
}
//-----------------------------------------------------------------------------------------------------------
Compare::YesListIterator Compare::getYesBegin(){
	return fYesList.begin();
}
//-----------------------------------------------------------------------------------------------------------
Compare::YesListIterator Compare::getYesEnd(){
	return fYesList.end();
}
//-----------------------------------------------------------------------------------------------------------
Compare::DiffVerListIterator Compare::getDiffVerBegin(){
	return fDiffVerList.begin();
}
//-----------------------------------------------------------------------------------------------------------
Compare::DiffVerListIterator Compare::getDiffVerEnd(){
	return fDiffVerList.end();
}
//-----------------------------------------------------------------------------------------------------------
File::FileIterator Compare::getNoBegin(){
	return fNoList.begin();
}
//-----------------------------------------------------------------------------------------------------------
File::FileIterator Compare::getNoEnd(){
	return fNoList.end();
}
//-----------------------------------------------------------------------------------------------------------
void Compare::clear() {
    Compare::YesListIterator YI = getYesBegin();
    Compare::DiffVerListIterator DI = getDiffVerBegin();
    Compare::MaybeListIterator MI = getMaybeBegin();
    File::FileIterator FI = getNoBegin();

    while (FI != getNoEnd()) {
	delete (*FI);
	++FI;
    }
    
    while (DI != getDiffVerEnd()) {
	FI = DI->second.begin();
	while (FI != DI->second.end()) {
	    delete (*FI);
	    ++FI;
	}
	if ( (fYesList.find(DI->first) != getYesEnd()) ||
 	     (fMaybeList.find(DI->first) != getMaybeEnd()) ) {
	    //It is perfectly possible for a track to simultaneously appear in the DiffVer and
	    //YesList or in the DiffVer and Maybe list (as 'first'), therfore this check (to prefevent segfaults 
	    //resulting from deleting the same pointer twice)
	    //cout<< "NOT DELETED " << *(DI->first) << "\n";
	} else {
	    delete (DI->first);
	}
	
	++DI;
    }
    
    while (MI != getMaybeEnd()) {
	FI = MI->second.begin();
	while (FI != MI->second.end()) {
	    delete (*FI);
	    ++FI;
	}
	delete (MI->first);
	
	++MI;
    }
    
    while (YI != getYesEnd()) {
	FI = YI->second.begin();
	while (FI != YI->second.end()) {
	    delete (*FI);
	    ++FI;
	}
	delete (YI->first);
	++YI;
    } 
  
    fYesList.clear();
    fNoList.clear();
    fMaybeList.clear();
    fDiffVerList.clear();
}
//-----------------------------------------------------------------------------------------------------------
void Compare::getKeywords(const QString s, QStringList& list) {
    //split the incoming QString into words (use space as separator)
    QString tmp = s;
    tmp = tmp.replace(QChar(','),QChar(' '));
    QStringList tmpList = QStringList::split(' ', tmp);
    list.clear();
    tmp = "";    
    
    for ( QStringList::Iterator it = tmpList.begin(); it != tmpList.end(); ++it ) {
	tmp = *it;
	//the mysql database "REGEXP '[[:<:]]word[[:>:]]'" expression doesnt work if "word" ends with punctuation (!,',...)
	tmp = stripTrailingPunct(tmp);
	tmp = Dbase::cleanSQL(tmp);
	if (fRcMan->isKeyword( tmp )) {
	    //the word is a keyword
	    list.append( tmp );
	    //cout<<"added keyword: " << (tmp) << "\n"; 
	} else {
	    //cout<<"not added keyword: " << (tmp) << "\n"; 
	}
    }
}
 //-----------------------------------------------------------------------------------------------------------
void Compare::compare(Track::Tracklist& tlist,Dbase* DB, const QString table,QProgressBar* pBar) {
    assert(DB->isOpen());
    
    int keywordCount = 0;
    int queryCount = 0;
    int trackCount = 0;
    
    File::Filelist reslist;
    File::Filelist maybelist;
    QStringList slist;
    QStringList slist2;
    
    QString tmp = "";
    QString query = "";
    
    QString oldArtist = "";
    QString oldTitle = "";
    QString oldComment = "";
    
    QString newArtist = "";
    QString newTitle = "";
    QString newComment = "";
    int newLength = 0;
    int newBitrate = 0;
    QString minLength = "";
    QString maxLength = "";
    bool found = false;
    
    pBar->reset();
    pBar->setTotalSteps(tlist.size());
    int pi = 0;
    
    try {
	try {
	    DB->dropTable("tmpTrack"); //if tmpTrack already exists remove it
	} catch(DbaseException* e) {
	}
	
	DB->createTmpTrackTable("tmpTrack");
	
	Track::ConstTrackIterator TI = tlist.begin();
	while (TI != tlist.end()) {
	    found = false;
	    reslist.clear();
	    
	    newArtist = Dbase::cleanSQL((*TI)->getArtist());
	    newTitle = Dbase::cleanSQL((*TI)->getTitle());
	    newComment = Dbase::cleanSQL((*TI)->getComment());
	    newLength = (*TI)->getLength();
	    newBitrate = (*TI)->getBitrate();
	    
	    minLength = tmp.setNum(newLength - fLenTolerance);
	    tmp = "";
	    maxLength = tmp.setNum(newLength + fLenTolerance);
	    
	    slist.clear();
	    slist2.clear();
	    getKeywords(newArtist,slist);
	    getKeywords(newTitle,slist2);
	    keywordCount += (slist.size() + slist2.size());
	    query = "INSERT INTO tmpTrack SELECT * FROM " + table 
		    + " AS T WHERE (" + DB->buildRegexpWhereClause("T.artist", slist) + " AND " 
		    + DB->buildRegexpWhereClause("T.title",slist2) + ");";
	    //cout<<"Initial query is: " << query << "\n";
	    
	    DB->query(query);
	    //tmpTrack now contains all tracks whose artist AND title overlap (in keywords) with the new track
	    
	    //now retrieve the YES-list, these are all database tracks which match exactly with the new track
	    //a database track matches the new track if artist AND title AND comment (optional) match and if the quality (length, bitrate, size,...)
	    //is comparable
	    
	    query = "(artist = \"" + newArtist + "\") AND (title = \"" + newTitle + "\")";
	    
	    if (fRequireEqualCom) {
		query += " AND (comment = \"" + newComment + "\")";
	    }
	    
	    //require comparable quality
	    if (newLength > 0) {  //only take into account length if we have that information
		//for 2 tracks to be equal their length may not differ more than a set tolerance
		query += " AND (  (length = 0) OR ( (length >= " + minLength + ") AND (length <= " + maxLength + ") )  )";
	    }
	    
	    if (newBitrate > 0) { //only take into account bitrate if we have that information
		//I already have the new track is the track in my database matches as stated above AND the quality (bitrate)
		// of my track is at least as good as the quality (bitrate) of the new track
		tmp = "";
		query += ( " AND (  (bitrate = 0) OR (bitrate >= " + tmp.setNum(newBitrate) + ")  )" );
	    }
	    
	    //sometimes artist and title ar switched, check for that case here
	    query = "(" + query + ") OR " + "((artist = \"" + newTitle + "\") AND (title = \"" + newArtist + "\"))";
	    
	    //The 'query' variable now determines the YES-list completely, we can now get the YES-list tracks
	    //cout<<"yeslist query is: " << query << "\n\n";
	    DB->queryTracks(query,reslist,"tmpTrack");
	    
	    if (!reslist.empty()) {
		//we now are sure we already have the particular track
		//no need to carry on querying, to be sure however we also check for different versions
		//before returning
		found = true;
		fYesList[(*TI)] = reslist;
		reslist.clear();
	    }
	    
	    //cout <<"Yes Reslist contains: \n";
	    //for ( File::ConstFileIterator it = reslist.begin(); it != reslist.end(); ++it ) cout<< "Name: " << (*it)->getName() + "\n";
	    
	    
	    //we have retrieved the tracks and no longer need them
	    DB->deleteFromTable(query, "tmpTrack");
	    
	    //Step 2 is the retrieving of the "Different version"-list.  These are the same tracks as above except that
	    //they differ in their quality-attributes
	    query = "(artist = \"" + newArtist + "\") AND (title = \"" + newTitle + "\")";
	    DB->queryTracks(query,reslist,"tmpTrack");
	    
	    //cout <<"Diff ver Reslist contains: \n";
	    // for ( File::ConstFileIterator it = reslist.begin(); it != reslist.end(); ++it ) cout<< "Name: " << (*it)->getName() + "\n";
	    
	    if (!reslist.empty()) {
		fDiffVerList[(*TI)] = reslist;
		reslist.clear();
	    }
	    
	    //we have retrieved the tracks and no longer need them
	    DB->deleteFromTable(query, "tmpTrack");
	    
	    if (!found) {
		//Step 3: If 'found' is still false we now continue our search for possible matches.
		// We now consider tracks whose title matches exactly and whose artist (keyword) overlaps
		query = "( (title = \"" + newTitle + "\") AND "+ DB->buildRegexpWhereClause("artist", slist) +")";
		DB->queryTracks(query,maybelist,"tmpTrack");
		
		for ( File::ConstFileIterator it = maybelist.begin(); it != maybelist.end(); ++it ) reslist.push_back(*it);
		maybelist.clear();
		
		//we have retrieved the tracks and no longer need them
		DB->deleteFromTable(query, "tmpTrack");
		
		
		//Step 4: We now consider tracks whose artist matches exactly and whose title (keyword) overlaps
		query = "( (artist = \"" + newArtist + "\") AND "+ DB->buildRegexpWhereClause("title", slist2) +")";
		DB->queryTracks(query,maybelist,"tmpTrack");
		
		for ( File::ConstFileIterator it2 = maybelist.begin(); it2 != maybelist.end(); ++it2 ) reslist.push_back(*it2);
		maybelist.clear();
		
		//we have retrieved the tracks and no longer need them
		DB->deleteFromTable(query, "tmpTrack");
		
		//All that now rests in tmpTrack are all tracks whose artist/title partially matches
		//with the new track.  Step 5 is to retrieve all of these.
		query = "";
		DB->queryTracks(query,maybelist,"tmpTrack");
		
		for ( File::ConstFileIterator it3 = maybelist.begin(); it3 != maybelist.end(); ++it3 ) reslist.push_back(*it3);
		maybelist.clear();
		
		//cout <<"maybe Reslist contains: \n";
		//for ( File::ConstFileIterator it = reslist.begin(); it != reslist.end(); ++it ) cout<< "Name: " << (*it)->getName() + "\n";
		
		if (!reslist.empty()) {
		    found = true;
		    fMaybeList[(*TI)] = reslist;
		}
		
		if (!found) {
		    fNoList.push_back(*TI);
		}
	    }
	    
	    DB->clearTable("tmpTrack");
	    ++TI;
	    ++pi;
	    pBar->setProgress(pi);
	}
	DB->dropTable("tmpTrack");
	
    } catch (DbaseException* e) {    
	throw( new Exception("Compare::compare","Database exception occured during compare, error is:\n" + e->msg()) );
    }
    
    QString tmp1 = "";
    QString tmp2 ="";
    trackCount = tlist.size();
    queryCount = (11 * trackCount) + 1;
    float keywordsPerTrack = keywordCount / trackCount;
    
    DomoGlobals::logger->info("Compare::compare","Compare done: " + tmp.setNum(trackCount) 
			      + " tracks, " + tmp1.setNum(queryCount) + " queries, " 
			      + tmp2.setNum(keywordsPerTrack) + " keywords per track (avg)");
}
//-----------------------------------------------------------------------------------------------------------
void Compare::displayResults(QListView* compareResults)  {
    Compare::YesListIterator YI = getYesBegin();
    Compare::DiffVerListIterator DI = getDiffVerBegin();
    Compare::MaybeListIterator MI = getMaybeBegin();
    File::FileIterator NI = getNoBegin();
    
    File::FileIterator FI;
    Track* trk = 0;
    QString tmp = "";
    
    while (NI != getNoEnd()) {
	//fLogger->info("Compare::","adding no-list item");
	QListViewItem* it = new QListViewItem(compareResults);
	it->setText(0, "------------");
	it->setText(1, (*NI)->getName());
	it->setText(2, "No" );
	//it->setText(3, "---");
	//it->setText(4, ((*NI).first)->getLength());
	//it->setText(5, ((*NI).first)->getBitrate());
	++NI;
    }
    
    while (DI != getDiffVerEnd()) {
	//fLogger->info("Compare::","adding diffver-list item");
	QListViewItem* it = new QListViewItem(compareResults);
	trk = TrackSpace::castToTrack(DI->first);
	assert(trk != 0);
	it->setText(0, "------------");
	it->setText(1, trk->getName());
	it->setText(2, "Different Version" );
	it->setText(3, "---");
	it->setText(4, tmp.setNum(trk->getLength()));
	it->setText(5, tmp.setNum(trk->getBitrate()));
	
	FI = DI->second.begin();
	while (FI != DI->second.end()) {
	    QListViewItem* subIt = new QListViewItem(it);
	    trk = TrackSpace::castToTrack(*FI);
	    assert(trk != 0);
	    
	    subIt->setText(0, tmp.setNum(trk->getID()));
	    subIt->setText(1, trk->getName());
	    subIt->setText(2, "---");
	    subIt->setText(3, tmp.setNum(trk->getCDno()));
	    subIt->setText(4, tmp.setNum(trk->getLength()));
	    subIt->setText(5, tmp.setNum(trk->getBitrate()));
	    
	    ++FI;
	}
	it->setOpen(true);
	++DI;
    }
    
    while (MI != getMaybeEnd()) {
	QListViewItem* it = new QListViewItem(compareResults);
	trk = TrackSpace::castToTrack(MI->first);
	assert(trk != 0);
	
	it->setText(0, "------------");
	it->setText(1, trk->getName());
	it->setText(2, "Maybe" );
	it->setText(3, "---");
	it->setText(4, tmp.setNum(trk->getLength()));
	it->setText(5, tmp.setNum(trk->getBitrate()));
	
	FI = MI->second.begin();
	while (FI != (*MI).second.end()) {
	    QListViewItem* subIt = new QListViewItem(it);
	    trk = TrackSpace::castToTrack(*FI);
	    assert(trk != 0);
	    
	    subIt->setText(0, tmp.setNum(trk->getID()));
	    subIt->setText(1, trk->getName());
	    subIt->setText(2, "---");
	    subIt->setText(3, tmp.setNum(trk->getCDno()));
	    subIt->setText(4, tmp.setNum(trk->getLength()));
	    subIt->setText(5, tmp.setNum(trk->getBitrate()));
	    
	    ++FI;
	}
	it->setOpen(true);
	++MI;
    }
    
    while (YI != getYesEnd()) {
	QListViewItem* it = new QListViewItem(compareResults);
	trk = TrackSpace::castToTrack(YI->first);
	assert(trk != 0);
	
	it->setText(0, "------------");
	it->setText(1, (YI->first)->getName());
	it->setText(2, "Yes" );
	it->setText(3, "---");
	it->setText(4, tmp.setNum(trk->getLength()));
	it->setText(5, tmp.setNum(trk->getBitrate()));
	
	FI = YI->second.begin();
	while (FI != YI->second.end()) {
	    QListViewItem* subIt = new QListViewItem(it);
	    trk = TrackSpace::castToTrack(*FI);
	    assert(trk != 0);
	    
	    subIt->setText(0, tmp.setNum(trk->getID()));
	    subIt->setText(1, trk->getName());
	    subIt->setText(2, "---");
	    subIt->setText(3, tmp.setNum(trk->getCDno()));
	    subIt->setText(4, tmp.setNum(trk->getLength()));
	    subIt->setText(5, tmp.setNum(trk->getBitrate()));
	    
	    ++FI;
	}
	++YI;
    }
}
//-----------------------------------------------------------------------------------------------------------
void Compare::exportCompareSelection(QListView* compareResults,
				     const QString filename,
				     const bool noEntries,
				     const bool diffVerEntries,
				     const bool maybeEntries,
				     const bool yesEntries,
				     const bool selectedEntries) {
    
    Compare::YesListIterator YI = getYesBegin();
    Compare::DiffVerListIterator DI = getDiffVerBegin();
    Compare::MaybeListIterator MI = getMaybeBegin();
    File::FileIterator NI = getNoBegin();
    File::FileIterator TI;
    
    try {	
	fLogger->info("Compare::exportCompareSelection","Exporting to '" + filename + "'");
	WriterTxt tw(filename);
	tw.setShort(true);
	
	tw.writeHeader( QObject::tr(	"\n###############################"
					"\n# DOMO Generated Compare File #"
					"\n###############################"
					"\n\n(Your corresponding tracks are marked with '*')\n") );
	
	if (noEntries) {
	    tw.write( QObject::tr("\nTracks you do NOT have\n"
				  "-----------------------\n\n") );
	    
	    while (NI != getNoEnd()) {
		tw.write(*NI);	
		tw.write("\n");	
		++NI;
	    }
	}
	
	if (diffVerEntries) {
	    tw.write(	QObject::tr("\n\nTracks of which you have a different version\n"
				    "--------------------------------------------\n\n") );
	    while (DI != getDiffVerEnd()) {
		tw.write((*DI).first);	
		
		TI = DI->second.begin();
		while (TI != (*DI).second.end()) {
		    tw.write("\t*" + (*TI)->getName() + '\n');
		    ++TI;
		}
		tw.write("\n");
		++DI;
	    }
	}
	
	if (maybeEntries) {
	    tw.write(	QObject::tr("\n\nTracks which you MIGHT have\n"
				    "---------------------------\n\n") );
	    while (MI != getMaybeEnd()) {
		tw.write((*MI).first);
		tw.write("\n");
		
		TI = MI->second.begin();
		while (TI != (*MI).second.end()) {
		    tw.write("\t*" + (*TI)->getName() + '\n');
		    ++TI;
		}
		tw.write("\n");
		++MI;
	    }
	}
	
	
	if (yesEntries) {
	    tw.write(	QObject::tr("\n\nTracks which you have\n"
				    "---------------------\n\n") );
	    while (YI != getYesEnd()) {
		tw.write((*YI).first);	
		tw.write("\n");
		
		TI = YI->second.begin();
		while (TI != (*YI).second.end()) {
		    tw.write("\t*" + (*TI)->getName() + '\n');
		    ++TI;
		}
		tw.write("\n");
		++YI;
	    }
	}
	
	if (selectedEntries) {
	    tw.write(	QObject::tr("\n\nTracks you selected\n"
				    "---------------------\n\n") );
	    QListViewItemIterator it(compareResults);
	    while(it.current()){
		if (it.current()->isSelected()){
		    tw.write( it.current()->text(1));
		    tw.write("\n");
		}
		++it;
	    }	
	}
	
	tw.writeFooter();
	
	fLogger->info("Compare::exportCompareSelection","Compare selection exported...");
	
    } catch (IoException* e) {
	fLogger->exception("Compare::exportCompareSelection",e->msg());
	fLogger->error("Compare::exportCompareSelection","Export compare selection failed...");
    }
}
//-----------------------------------------------------------------------------------------------------------
void Compare::deleteCompareSelection(QListView* compareResults,
				     const QString comparePath,
				     const bool noEntries, 
				     const bool diffVerEntries,
				     const bool maybeEntries,
				     const bool yesEntries,
				     const bool selectedEntries) {
    
    Compare::YesListIterator YI = getYesBegin();
    Compare::DiffVerListIterator DI = getDiffVerBegin();
    Compare::MaybeListIterator MI = getMaybeBegin();
    File::FileIterator NI = getNoBegin();
    File::FileIterator TI;
    
    YI = getYesBegin();
    DI = getDiffVerBegin();
    MI = getMaybeBegin();
    NI = getNoBegin();
    QString path = "";
    QString fname = "";
    
    if (noEntries) {
	while (NI != getNoEnd()) {
	    path = (*NI)->getPath();
	    fname = path + DomoGlobals::DIR_SEP + (*NI)->getName();
	    if (FileIO::deleteFile(fname)){
		fLogger->info("Compare::deleteCompareSelection",fname + " removed...");
	    } else {
		fLogger->warning("Compare::deleteCompareSelection",fname + " remove FAILED...");
	    }
	    ++NI;
	}
    }
    
    if (diffVerEntries) {
	while (DI != getDiffVerEnd()) {
	    path = (*DI).first->getPath();
	    fname = path + DomoGlobals::DIR_SEP + (*DI).first->getName();
	    if (FileIO::deleteFile(fname)){
		fLogger->info("Compare::deleteCompareSelection",fname + " removed...");
	    } else {
		fLogger->warning("Compare::deleteCompareSelection",fname + " remove FAILED...");
	    }
	    ++DI;
	}
    }
    
    if (maybeEntries) {
	while (MI != getMaybeEnd()) {
	    path = (*MI).first->getPath();
	    fname = path + DomoGlobals::DIR_SEP + (*MI).first->getName();
	    if (FileIO::deleteFile(fname)){
		fLogger->info("Compare::deleteCompareSelection",fname + " removed...");
	    } else {
		fLogger->warning("Compare::deleteCompareSelection",fname + " remove FAILED...");
	    }
	    ++MI;
	}
    }
    
    
    if (yesEntries) {
	while (YI != getYesEnd()) {
	    path = (*YI).first->getPath();
	    fname = path + DomoGlobals::DIR_SEP + (*YI).first->getName();
	    if (FileIO::deleteFile(fname)){
		fLogger->info("Compare::deleteCompareSelection",fname + " removed...");
	    } else {
		fLogger->warning("Compare::deleteCompareSelection",fname + " remove FAILED...");
	    }
	    
	    ++YI;
	}
    }
    
    if (selectedEntries) {
	QListViewItemIterator it( compareResults );
	path = comparePath;
	while ( it.current() ) {
	    if ( it.current()->isSelected() ) {
		fname = path + DomoGlobals::DIR_SEP + it.current()->text(1);
		
		if (FileIO::deleteFile(fname)){
		    fLogger->info("Compare::deleteCompareSelection",fname + " removed");
		} else {
		    fLogger->warning("Compare::deleteCompareSelection",fname + " remove FAILED");
		}
	    }	
	    ++it;
	}
    }
    
    fLogger->info("Compare::deleteCompareSelection","Delete selection done...");
}
//-----------------------------------------------------------------------------------------------------------
