/* --------------------------------------------------------------------------
   
   Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)

   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation;
   
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this application; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----------------------------------------------------------------------------*/
#include "metainfo.h"
#include "exception.h"
#include "stringmanip.h"
#include "assert.h"
#include "qstringlist.h"
#include "domoglobals.h"
#include "fileio.h"
#include "taglib.h"
#include "fileref.h"
#include "tag.h"
#include "tstring.h"
#include "audioproperties.h" 
#include "mpegproperties.h"
#include "vorbisproperties.h"
#include "musicexpression.h"

#include <stdio.h>
#include <string>
#include <iostream>
using std::cout;
using std::string;
using TagLib::FileRef;
using TagLib::Tag;
using TagLib::AudioProperties;
//-----------------------------------------------------------------------------------------------------------
MetaInfo::MetaInfo() {
    
}
//-----------------------------------------------------------------------------------------------------------
MetaInfo::~MetaInfo() {
    
}
//-----------------------------------------------------------------------------------------------------------
void MetaInfo::getMetaInfo(Track* trk, const bool override) {
    assert(trk != 0);
    
    DomoGlobals::logger->debug("MetaInfo::getMetaInfo","Getting metainfo for file '" + trk->absPath() + "'");

    ContentType t = getContentType(trk->getName());

    switch (t) {
	case MP3: {
	    getMp3Info(trk, override);
	    break;
	}
	case OGG: {
	    getOggInfo(trk, override);
	    break;
	}
	case WMA:{
	    getWmaInfo(trk, override);
	    break;
	}
	case WAV:{
	    getWavInfo(trk, override);
	    break;
	}
	default: {
	    break;
	}
    }
}
//-----------------------------------------------------------------------------------------------------------
Track*  MetaInfo::getMetaInfo(QString path, const bool override) {
	QString fname = "";
	QString dir = "";
	FileIO::splitAbsPath(path, dir, fname);
	Track* t = new Track();
	t->setName(fname);
	t->setPath(dir);
	getMetaInfo(t,override);
	return t;
}
//-----------------------------------------------------------------------------------------------------------
void MetaInfo::getMp3Info(Track* t, const bool override) {
    assert(t != 0);
    QString path = t->absPath();
    
    if (!QFile::exists(path)) {
     	DomoGlobals::logger->warning("MetaInfo::getMp3Info","Cannot find file '" + path + "'");
     	return;
    }
 
    TagLib::FileRef f(path.latin1());
    if (f.isNull()) {
     	DomoGlobals::logger->warning("MetaInfo::getMp3Info","Cannot get metatag from '" + path + "'");
     	return;
    }
    
    QString artist = f.tag()->artist().toCString();
    QString title = f.tag()->title().toCString();
    QString comment = f.tag()->comment().toCString();
    QString year = ""; year.setNum(f.tag()->year());
    QString album = f.tag()->album().toCString();
    QString genre = f.tag()->genre().toCString();
    int trackNo = f.tag()->track();
    
    AudioProperties* audioProps = f.audioProperties();
    assert(audioProps != 0);
    
    int brate = audioProps->bitrate();
    int sampleRate = audioProps->sampleRate();
    int len = audioProps->length();
    int channels = audioProps->channels();
    
    //cast to subclass for mpeg specific information
    TagLib::MPEG::Properties* mpegProps = dynamic_cast<TagLib::MPEG::Properties*>(audioProps);
    assert(mpegProps != 0);
    
    int mode = mpegProps->channelMode();
    int version = mpegProps->version();
    int layer = mpegProps->layer();
    //Causes undefined reference for some reason
    //bool protection = mpegProps->protectionEnabled();
    bool copyrighted = mpegProps->isCopyrighted();
    bool original = mpegProps->isOriginal();
   
    t->setBitrate(brate);
    t->setSamplingRate(sampleRate);
    t->setLength(len);   
   
    if(override) {
	    //override filename data if non-empty metadata exists
	    if(!empty(comment)) t->setComment(comment);
	    if(!empty(artist)) t->setArtist(artist);
	    if(!empty(title)) t->setTitle(title);
	    if(!empty(genre)) t->setGenre(genre);
	    if(!empty(album)) t->setAlbum(album);
	    if(year > 0) t->setYear(year);
	    if(trackNo > 0) t->setTrackNo(trackNo);
    } else {
	    //only override empty filename-derived data
	    if(!empty(comment)) t->appendComment(comment.stripWhiteSpace());
	    if(empty(t->getAlbum())) t->setAlbum(album);
	    if(empty(t->getArtist())) t->setArtist(artist);
	    if(empty(t->getTitle())) t->setTitle(title);
	    if(empty(t->getGenre())) t->setGenre(genre);
	    if(t->getYear() <= 0) t->setYear(year);
	    if(t->getTrackNo() <= 0) t->setTrackNo(trackNo);
    }
}
//-----------------------------------------------------------------------------------------------------------
void MetaInfo::getOggInfo(Track* t, const bool override) {
    assert(t != 0);
    QString path = t->absPath();
    
    if (!QFile::exists(path)) {
     	DomoGlobals::logger->warning("MetaInfo::getOggInfo","Cannot find file '" + path + "'");
     	return;
    }
 
    TagLib::FileRef f(path);
    
    if (f.isNull()) {
     	DomoGlobals::logger->warning("MetaInfo::getOggInfo","Cannot get metatag from '" + path + "'");
     	return;
    }
		   
    QString artist = f.tag()->artist().toCString();
    QString title = f.tag()->title().toCString();
    QString comment = f.tag()->comment().toCString();
    QString year = ""; year.setNum(f.tag()->year());
    QString album = f.tag()->album().toCString();
    QString genre = f.tag()->genre().toCString();
    int trackNo = f.tag()->track();
    
    AudioProperties* audioProps = f.audioProperties();
    assert(audioProps != 0);
    
    int bitrate = audioProps->bitrate();
    int sampleRate = audioProps->sampleRate();
    int length = audioProps->length();
    int channels = audioProps->channels();
    
    //cast to subclass for ogg specific information
    TagLib::Vorbis::Properties* oggProps = dynamic_cast<TagLib::Vorbis::Properties*>(audioProps);
    assert(oggProps != 0);
    
    int version = oggProps->vorbisVersion();
    int maxBitrate = oggProps->bitrateMaximum();
    int nominalBitrate = oggProps->bitrateNominal();
    int minBitrate = oggProps->bitrateMinimum();
    
    t->setBitrate(bitrate);
    t->setSamplingRate(sampleRate);
    t->setLength(length);

    if(override) {
	    //override filename data if non-empty metadata exists
	    if(!empty(comment)) t->setComment(comment);
	    if(!empty(artist)) t->setArtist(artist);
	    if(!empty(title)) t->setTitle(title);
	    if(!empty(genre)) t->setGenre(genre);
	    if(!empty(album)) t->setAlbum(album);
	    if(year > 0) t->setYear(year);
	    if(trackNo > 0) t->setTrackNo(trackNo);
    } else {
	    //only override empty filename-derived data
	    if(!empty(comment)) t->appendComment(comment.stripWhiteSpace());
	    if(empty(t->getAlbum())) t->setAlbum(album);
	    if(empty(t->getArtist())) t->setArtist(artist);
	    if(empty(t->getTitle())) t->setTitle(title);
	    if(empty(t->getGenre())) t->setGenre(genre);
	    if(t->getYear() <= 0) t->setYear(year);
	    if(t->getTrackNo() <= 0) t->setTrackNo(trackNo);
    }
}
//-----------------------------------------------------------------------------------------------------------
void MetaInfo::getWmaInfo(Track* t, const bool override) {
    assert(t != 0);
    //to get rid of compiler warnings
    if (override) {
    }
}
//-----------------------------------------------------------------------------------------------------------
void MetaInfo::getWavInfo(Track* t, const bool override) {
    assert(t != 0);
    //to get rid of compiler warnings
    if (override) {
    }
}
//-----------------------------------------------------------------------------------------------------------
void MetaInfo::writeMetaInfo(const Track* t) {
     assert(t != 0);
    QString path = t->absPath();
    
    if (!QFile::exists(path)) {
     	DomoGlobals::logger->warning("MetaInfo::writeMetaInfo","Cannot find file '" + path + "'");
     	return;
    }
    
    if (!FileIO::isWriteable(path.latin1())) {
     	DomoGlobals::logger->warning("MetaInfo::writeMetaInfo","File '" + path + "' is not writeable.");
     	return;
    } 
    TagLib::FileRef f(path);
    
     if (f.isNull()) {
     	DomoGlobals::logger->warning("MetaInfo::writeMetaInfo","Cannot get metatag from '" + path + "'");
     	return;
    }
     
    if(!t->getAlbum().isEmpty()) f.tag()->setAlbum(t->getAlbum().latin1());
    if(!t->getArtist().isEmpty()) f.tag()->setArtist(t->getArtist().latin1());
    if(!t->getTitle().isEmpty()) f.tag()->setTitle(t->getTitle().latin1());
    if(t->getYear() > 0) f.tag()->setYear(t->getYear());
    if(!t->getComment().isEmpty() )f.tag()->setComment(t->getComment().latin1());
    if(!t->getGenre().isEmpty()) f.tag()->setGenre(t->getGenre().latin1());
    if(t->getTrackNo() > 0) f.tag()->setTrack(t->getTrackNo());
 
    if (f.save()) {
     	DomoGlobals::logger->info("MetaInfo::writeMetaInfo","Metatag written to '" + path + "'");
    } else {
     	DomoGlobals::logger->warning("MetaInfo::writeMetaInfo","Failed to write metatag to '" + path + "'");
    }
    
}
//-----------------------------------------------------------------------------------------------------------
QString MetaInfo::filenameFromTag(const QString filepath, const QString fnameExpression) {
	Track* t = MetaInfo::getMetaInfo(filepath,true);
	//cout<<"comment is " << t->getComment() << "\n";
	return MusicExpression::makeFilename(t,fnameExpression);
}
//-----------------------------------------------------------------------------------------------------------
void MetaInfo::writeTagsFromFilename(const QString filepath, const QString expression) {
	QString fname = "";
	QString dir = "";
	FileIO::splitAbsPath(filepath, dir, fname);	
	
	Track* t = MusicExpression::makeTrack(expression, fname);
	t->setName(fname);
	t->setPath(dir);
	MetaInfo::writeMetaInfo(t);
}
//-----------------------------------------------------------------------------------------------------------

