/*-----------------------------------------------------------------------
QMBTagger - Qt Based MusicBrainz MP3/OGG Tagger
Copyright (C) 2003  Ewen Cheslack-Postava

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-----------------------------------------------------------------------*/

#include "qmbfile.h"
#include <vector>
#include <qstring.h>
#include <qapplication.h>
#include <musicbrainz/musicbrainz.h>
#include <musicbrainz/mb_c.h>
#include <musicbrainz/queries.h>

QMBFile::QMBFile()
	:bufferlen(0), buffer(0), channels(0), sampleRate(0), songlength(0), queryresults(NULL)
{
	//we do nothing here.  we want to allow checking of return value,
	//so we only allow class to function through calling of Extract()
}

//this is a pure virtual destructor, so we obviously don't do anything
QMBFile::~QMBFile()
{

}
//returns:
//	-2: failed Finalizing signature
//	-1: failed GenerateSignature
//	0: success
int QMBFile::CreateID(trm_t &trm) {

	//set the data we got from Extract()
	trm_SetPCMDataInfo(trm, sampleRate, channels, 16);

	//set the song length (in seconds) found in Extract()
	trm_SetSongLength(trm, (long int)songlength);

	//try to generate the signature
	if(trm_GenerateSignature(trm, buffer, bufferlen) == 0) {
		//failed the GenerateSignature because more data was needed
		qWarning("QMBFile::CreateID(): Failed to generate signature - more data was needed (%s).", fullfilename.latin1());
		trm_Delete(trm);
		return -1;
	} else {
		//signature successful and was given enough data
		if(trm_FinalizeSignature(trm, signature, NULL) == -1) {
			//finalize signature failed, bail out gracefully
			qWarning("QMBFile::CreateID(): Failed to finalize signature (%s).", fullfilename.latin1());
			trm_Delete(trm);
			return -2;
		}
	}

	//now just get it into ASCII form
	trm_ConvertSigToASCII(trm, signature, ascii_sig);
	//now copy this into the metadata
	metadata.trmid = ascii_sig;

	if (buffer != 0) {
		delete [] buffer;
		bufferlen = 0;
		buffer = NULL;
	}
	return 0;
}

//returns:
//-1 on error
// otherwise the number of queries returned
int QMBFile::QueryServer(MusicBrainz* mb) {

	//now get ready for the actual query
	vector<std::string> queryargs(6);

	//set the query args
	queryargs[0] = metadata.trmid.latin1();
	queryargs[1] = metadata.artistName.latin1();
	queryargs[2] = metadata.albumName.latin1();
	queryargs[3] = metadata.trackName.latin1();
	queryargs[4] = metadata.trackNum.latin1();
	queryargs[5] = metadata.duration.latin1();

	//run the query
	if (!mb->Query(MBQ_TrackInfoFromTRMId,&queryargs)) {
		std::string queryerror; //FIXME: we want this to use QString if possible
		mb->GetQueryError(queryerror);
		qWarning("QMBFile::QueryServer(): MBQ_TrackInfoFromTRMId on file (%s) failed: %s", fullfilename.latin1(), queryerror.c_str());
		return -1;
	}

	numresults = mb->DataInt(MBE_GetNumTracks);

	if (numresults < 1) {
		qWarning("QMBFile::QueryServer(): MBQ_TrackInfoFromTRMId on file (%s) returned no results, trying MBQ_FileInfoLookup.", fullfilename.latin1());

		//setup the query args for MBQ_FileInfoLookup
		vector<std::string> filequeryargs(11);
		filequeryargs[0] = metadata.trmid.latin1();
		filequeryargs[1] = metadata.artistName.latin1();
		filequeryargs[2] = metadata.albumName.latin1();
		filequeryargs[3] = metadata.trackName.latin1();
		filequeryargs[4] = metadata.trackNum.latin1();
		filequeryargs[5] = metadata.duration.latin1();
		filequeryargs[6] = metadata.fileName.latin1();
		filequeryargs[7] = metadata.artistId.latin1();
		filequeryargs[8] = metadata.albumId.latin1();
		filequeryargs[9] = metadata.trackId.latin1();
		filequeryargs[10] = "10"; //maxresults

		if (!mb->Query(MBQ_FileInfoLookup, &filequeryargs)) {
			std::string queryerror; //FIXME: we want this to use QString if possible
			mb->GetQueryError(queryerror);
			qWarning("QMBFile::QueryServer(): MBQ_FileInfoLookup on file (%s) failed: %s", fullfilename.latin1(), queryerror.c_str());
			return -1;
		}

		numresults = mb->DataInt(MBE_GetNumTracks);

		if (numresults < 1) {
			qWarning("QMBFile::QueryServer(): Couldn't find any results for file (%s).", fullfilename.latin1());
			return -1;
		}
	}

	queryresults = new QMBMetadata[numresults];

	for (int selecttrack = 1; selecttrack <= numresults; selecttrack++) {
		mb->Select(MBS_Rewind);
		mb->Select(MBS_SelectTrack, selecttrack);
		queryresults[selecttrack-1].trackName = (QString)mb->Data(MBE_TrackGetTrackName).c_str();
		queryresults[selecttrack-1].trackId = (QString)mb->Data(MBE_TrackGetTrackId).c_str();
		QString trackIdURI = queryresults[selecttrack-1].trackId; //save so we can use the URI to get the trackNum
		this->StripURL(queryresults[selecttrack-1].trackId);
		
		queryresults[selecttrack-1].duration = (QString)mb->Data(MBE_TrackGetTrackDuration).c_str();
		queryresults[selecttrack-1].artistName = (QString)mb->Data(MBE_TrackGetArtistName).c_str();
		queryresults[selecttrack-1].artistId = (QString)mb->Data(MBE_TrackGetArtistId).c_str();
		this->StripURL(queryresults[selecttrack-1].artistId);

		mb->Select(MBS_SelectTrackAlbum);
		queryresults[selecttrack-1].albumName = (QString)mb->Data(MBE_AlbumGetAlbumName).c_str();
		queryresults[selecttrack-1].albumId = (QString)mb->Data(MBE_AlbumGetAlbumId).c_str();
		this->StripURL(queryresults[selecttrack-1].albumId);
		queryresults[selecttrack-1].trackNum 
			= QString::number(mb->GetOrdinalFromList(MBE_AlbumGetTrackList, trackIdURI.latin1()));

		//finally just set the trmid from the value we already have
		queryresults[selecttrack-1].trmid = metadata.trmid;
	}

	return numresults;
}

void QMBFile::CopySelectedMetadata(int resultChoice) {
	this->metadata.trmid = queryresults[resultChoice].trmid;
	this->metadata.artistName = queryresults[resultChoice].artistName;
	this->metadata.albumName = queryresults[resultChoice].albumName;
	this->metadata.trackName = queryresults[resultChoice].trackName;
	this->metadata.trackNum = queryresults[resultChoice].trackNum;
	this->metadata.duration = queryresults[resultChoice].duration;

	this->metadata.artistId = queryresults[resultChoice].artistId;
	this->metadata.albumId = queryresults[resultChoice].albumId;
	this->metadata.trackId = queryresults[resultChoice].trackId;
}

void QMBFile::StripURL(QString &url) {
	int lastslash = url.findRev("/");

	url = url.right(url.length() - lastslash - 1);
}
