/* --------------------------------------------------------------------------
    Domo - Dirks (Oberon) Music Organizer (http://domo.sf.net)
   Copyright (C) 2001 Dirk Gorissen, All Rights Reserved
   
12/2003 adapted from:
   
QMBTagger - Qt Based MusicBrainz MP3/OGG Tagger
Copyright (C) 2003  Ewen Cheslack-Postava

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-----------------------------------------------------------------------*/

#include "qmbfileview.h"
#include "qmbfile.h"
#include "qmbvorbisfile.h"
#include "qmbmp3file.h"
#include <qlistview.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qintdict.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qprogressdialog.h>
#include <qsettings.h>
#include <musicbrainz/musicbrainz.h>
#include <musicbrainz/mb_c.h>

QMBFileView::QMBFileView(QWidget* parent, const char* name, WFlags f)
	:QListView(parent, name, f), format("{ARTIST} - {TRACK}.{EXT}"), newdir(""), crossfs(true), files(1009), submit(), authenticated(false), username(""), password("")
{
	addColumn("Name");
	addColumn("Number");
	addColumn("Artist");
	addColumn("Album");
	addColumn("Title");
	addColumn("TrackNum");
	setSelectionMode(Multi);
	setRootIsDecorated(true);

	mb = new MusicBrainz();

	//set the default server info
	mb->SetServer("musicbrainz.org", 80);

	//set it to not use UTF8
	mb->UseUTF8(FALSE);

	trm = trm_New();

	connect(this, SIGNAL(SaveTRMs()), &submit, SLOT(Save()));
	connect(this, SIGNAL(LoadTRMs()), &submit, SLOT(Load()));

	return;
}

QMBFileView::~QMBFileView()
{
	//delete the musicbrainz object
	if (mb != NULL)
		delete mb;

	//delete the trm object
	trm_Delete(trm);

	//delete all the files we have created
	int filecount = filenames.count(), fileindex;
	QMBFile* curfile = NULL;
	for (fileindex = 0; fileindex < filecount; fileindex++) {
		curfile = files[fileindex];
		files.remove(fileindex);
		if (curfile != NULL)
			delete curfile;
	}

}

int QMBFileView::NumSavedTRMs() {
	return submit.Count();
}

bool QMBFileView::Authenticate() {
	if (this->authenticated) return true;

	//no need to actually store the username and password, just authenticate and warn if something is wrong
	if (!this->username.isEmpty() && !this->password.isEmpty() && !mb->Authenticate((std::string)this->username.latin1(), (std::string)this->password.latin1())) {
		qWarning("QMBFileView::Authenticate(): Authentication failed.");
		QMessageBox::information(this, "Authentication error...", "Failed to authenticate with the server.  Check your username.", QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);
		return false;
	}

	this->authenticated = true;
	return true;
}

void QMBFileView::TransmitSettings(QSettings* newsettings) {
	this->submit.SetSettings(newsettings);
}

void QMBFileView::SetFilenames(const QStringList &filelist) {

	int fileindex, filecount = filenames.count(), newfilecount = filelist.count();
	int totalfilecount = filecount + newfilecount;

	if (newfilecount < 1) {
		QMessageBox::information( this, "QMBTagger - File Selection", "No files were selected or the directory selected contained no supported files.");
		return;
	}

	//add the new items
	for( fileindex = filecount; fileindex < totalfilecount; fileindex++) {
		QMBFile* curfile = NULL;

		filenames.append(filelist[fileindex - filecount]);
		QString curfilename = filenames[fileindex];

		QString shortfilename = curfilename.right( curfilename.length() - curfilename.findRev("/") - 1);

		if (curfilename.right(3).upper() == "OGG") {
			curfile = new QMBVorbisFile();
		} else if (curfilename.right(3).upper() == "MP3") {
			curfile = new QMBMP3File();
		} else {
		      qWarning("QMBFileView::SetFilenames(): The current file (%s) doesn't appear to be supported.", curfilename.latin1());
		      //TODO: causes segfault
		      continue;
		}

		//insert into the file dictionary
		files.insert(fileindex, curfile);

		//add the parent level item without any information
		QListViewItem* toplevel = new QListViewItem(this, shortfilename, QString::number(fileindex), "", "", "", "");
		toplevel->setSelectable(FALSE);
		toplevel->setOpen(TRUE);

		//insert into the list dictionary
		listparents.insert(fileindex, toplevel);
	}

}

/*Adapted by Dirk Gorissen 12/2003 for domo (http://domo.sf.net) */
void QMBFileView::QueryServer() {

	if (filenames.isEmpty()) {
		qWarning("QMBFileView::QueryServer(): No filenames selected for tagging.");
		return;
	}

	int fileindex, filecount = filenames.count();

	//setup the progress dialog
	QProgressDialog progress("Querying server...", 0, 3*filecount, this, "progress", TRUE);
	//make the progress dialog show no matter how long it takes
	progress.setMinimumDuration(0);

	for( fileindex = 0; fileindex < filecount; fileindex++) {

		QMBFile* curfile = files[fileindex];
		QListViewItem* toplevel = listparents[fileindex];

		if (toplevel->childCount() > 0) continue; //don't process files that already have children, we know they have already been queried

		QString curfilename = filenames[fileindex];

		QString shortfilename = curfilename.right( curfilename.length() - curfilename.findRev("/") - 1);

		//the following three steps are just using the
		//functionality of the QMBFile class
		if (curfile->metadata.trmid.isEmpty()) { //only extract and create ids for files that don't have ids yet
			if (curfile->Extract(curfilename) < 0) {
				qWarning("QMBFileView::QueryServer(): The current file (%s) couldn't be extracted.", curfilename.latin1());
				continue;
			}

			emit ExtractedFileData("Extracted Data From: " + shortfilename);
			//increment the progress dialog
			progress.setProgress(3*fileindex + 1);
			qApp->processEvents();

			if (curfile->CreateID(trm) < 0) {
				qWarning("QMBFileView::QueryServer(): The current file (%s) couldn't have an ID properly made.", curfilename.latin1());
				continue;
			}

			emit CreatedId("Created ID for: " + shortfilename);
			//increment the progress dialog
			progress.setProgress(3*fileindex + 2);
			qApp->processEvents();
		}

		int numtracks = curfile->QueryServer(mb);
		emit QueriedServer("Queried Server for: " + shortfilename);
		qApp->processEvents();
		if (numtracks < 1) {
			qWarning("QMBFileView::QueryServer(): The current file (%s) did not successfully get results from the server.", curfilename.latin1());
		}

		//fill in the rest of the information to the parent level item
		toplevel->setText(2, curfile->metadata.artistName);
		toplevel->setText(3, curfile->metadata.albumName);
		toplevel->setText(4, curfile->metadata.trackName);
		toplevel->setText(5, curfile->metadata.trackNum);

		//now we just add all the results
		QListViewItem* child = 0;
		for (int resultindex = 0; resultindex < numtracks; resultindex++) {
			child = new QListViewItem(toplevel, "RESULT " + QString::number(resultindex), QString::number(resultindex), curfile->queryresults[resultindex].artistName, curfile->queryresults[resultindex].albumName, curfile->queryresults[resultindex].trackName, curfile->queryresults[resultindex].trackNum);
		}

		emit AddedResultsToList(numtracks);
		emit AddedResultsToList();

		//increase the progress
		progress.setProgress(3*(fileindex + 1));

		qApp->processEvents();

	}

	emit FinishedQuerying("Finished Querying.");

	return;
}

/*Added by Dirk Gorissen 12/2003 for domo (http://domo.sf.net) */
bool QMBFileView::getFirstSelected(QString& artist, QString& title, QString& album) const {
    QListViewItem* selItem = this->selectedItem();
    if (selItem != 0) {
	artist = selItem->text(2);
	title = selItem->text(4);
	album = selItem->text(3);
	return true;
    } else {
	return false;
    }
}

/*Added by Dirk Gorissen 12/2003 for domo (http://domo.sf.net) */
bool QMBFileView::addSelectedFileForSubmission() {
    QString tmp = "";
    QListViewItem* current = this->selectedItem();
    if (current != 0) {
	//get the index of the file in the QMBFileView::files
	QListViewItem* parent = current->parent();
	int fileindex = parent->text(1).toInt();
	QMBFile* selfile = files[fileindex];
	
	int resultChoice = current->text(1).toInt();	
	selfile->CopySelectedMetadata(resultChoice);
	
	//add the item to submit to the server
	submit.Add(selfile->metadata.trackId, selfile->metadata.trmid);
	//qWarning("Added " + selfile->metadata.trackId + "|" + selfile->metadata.trackName);
	return true;
    } else {
	return false;
    }
}

void QMBFileView::OutputSelected() {

	QListViewItem* current = this->firstChild();

	//setup the progress dialog
	QProgressDialog progress("Querying server...", 0, this->childCount(), this, "progress", TRUE);
	//make the progress dialog show no matter how long it takes
	progress.setMinimumDuration(0);

	while(current != 0) {
		if (current->isSelected() == TRUE) {
			//get the index for the query array in QMBFile::queryresults
			int queryindex = current->text(1).toInt();

			//get the index of the file in the QMBFileView::files
			QListViewItem* parent = current->parent();
			int fileindex = parent->text(1).toInt();

			QMBFile* selfile = files[fileindex];
			
			//now actually do the outputting
			if (selfile->Output(queryindex, format, newdir, crossfs) != 0) {
				//we errored out somewhere in the output functions
				qWarning("QMBFileView::OutputSelected(): Error outputting file (%s).", files[fileindex]->fullfilename.latin1());
			} else {
				//add the item to submit to the server
				submit.Add(selfile->metadata.trackId, selfile->metadata.trmid);

				//get to the next toplevel item
				while (current != 0 && current->parent() != 0) {
					current = current->itemBelow();
				}

				//now remove the parent item
				this->takeItem(parent);

				emit OutputtedFile("Outputted new file for: " + filenames[fileindex]);
			}
		}
		if (current != 0) current = current->itemBelow();
		//increase the progress
		progress.setProgress(progress.progress() + 1);
		qApp->processEvents();
	}

	emit FinishedOutputting("Finished writing files.");

	//nothing left to do, if we are here we must have succeeded
	return;
}

void QMBFileView::setOptions(QString server, int port, QString proxy, int pport, QString filenameformat, QString outputdir, bool copycrossfs, QString user, QString pass) {
	//store the filename format so it can be passed to the output function later
	this->format = filenameformat;

	//the serverurl and serverport are set immediately in the MusicBrainz object
	if (this->mb != NULL) {
		if (!server.isEmpty())
			mb->SetServer(server.latin1(), port);
		if (!proxy.isEmpty()) {
			mb->SetProxy(proxy.latin1(), (short)pport);
			trm_SetProxy(trm, (char*)proxy.latin1(), (short)pport);
		}
	}

	//store the output directory so it can be used in the output function
	this->newdir = outputdir;

	this->crossfs = copycrossfs;

	//store the username and password here
	if (this->username != user || this->password != pass) {
		this->username = user;
		this->password = pass;

		this->authenticated = false;

		this->Authenticate();
	}
}

void QMBFileView::SubmitTRMs() {

	this->Authenticate();

	if (submit.Submit(mb)) {
		emit SubmittedTRMs("Submitted TRMs");
	} else {
		QMessageBox::information(this, "Submission error...", "Couldn't submit TRMs.  Check your password.", QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);
	}
}
