/*-----------------------------------------------------------------------
QMBTagger - Qt Based MusicBrainz MP3/OGG Tagger
Copyright (C) 2003  Ewen Cheslack-Postava

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-----------------------------------------------------------------------*/

//Changed by Dirk Gorissen <dgorissen@gmx.net> on 19/8/2004
//Removed ID3Lib dependencies, replaced by taglib

#include "qmbmp3file.h"
#include "qmbfile.h"
#include <stdio.h>
#include <string.h>
#include <qapplication.h>
#include <qstring.h>
#include <musicbrainz/musicbrainz.h>
#include "qmbmad.h"
#include <fstream>
#include "fileio.h"
#include "taglib.h"
#include "fileref.h"
#include "tag.h"
#include "tstring.h"
#include "audioproperties.h" 
#include "mpegproperties.h"
#include "vorbisproperties.h"

using std::string;
using TagLib::FileRef;
using TagLib::Tag;
using TagLib::AudioProperties;

QMBMP3File::QMBMP3File()
	: QMBFile()
{
	//we do nothing here.  we want to allow checking of return value,
	//so we only allow class to function through calling of Extract()
}

QMBMP3File::~QMBMP3File()
{
	if (buffer != 0) {
		delete [] buffer;
		bufferlen = 0;
	}
	if (queryresults != NULL) {
		delete [] queryresults;
	}
}

//get the data we want from the vorbis stream
//heavily borrowed code from vorbis doc on decoding vorbis stream
//returns:
//	0  indicates success
//	-1 indicates failed to open
//	-2 indicates invalid info
//	-3 indicates decoder failure
//	1 indicates success, but metadata extraction failed
int QMBMP3File::Extract(QString filename) {
	FILE* file = NULL;
 	int bitrate, durationmilli, samplesint;
	int ret = 0;

	//record the full filename so we will be able to use it for output
	fullfilename = filename;

	//set the filename in the metadata
	int lastslash = filename.findRev('/');
	metadata.fileName = filename.right(filename.length() - lastslash - 1);

	//get the file's info
	MusicBrainz mb;
	std::string strfilename = filename.latin1(); //FIXME: change this to use QString if possible
	if (mb.GetMP3Info(strfilename, durationmilli, bitrate, channels, samplesint)) {
		metadata.duration = durationmilli;
		songlength = durationmilli/1000.;
		channels++; //this is because GetMP3Info reports single channel as 0 and stereo as 1
		sampleRate = samplesint;
	} else {
		qWarning("QMBMP3File::Extract(): GetMP3Info failed on file (%s).", fullfilename.latin1());
		return -2;
	}

	//if the info is invalid, return without doing anything
	if (channels == 0 || sampleRate == 0) {
		qWarning("QMBMP3File::Extract(): Channels (%d) or SampleRate (%ld) not set properly for file (%s).", channels, sampleRate, fullfilename.latin1());
		return -2;
	}

	//get metadata
	if (GetMetadata(fullfilename) == -1) {
		//getting metadata failed - very strange! indicate with return val
		qWarning("QMBMP3File::Extract(): GetMetadata failed: File (%s) must be corrupted.", fullfilename.latin1());
		ret = 1;
	}

	//now for the actual decoding
	file = fopen(fullfilename.latin1(), "r");
	if (file == NULL) {
		qWarning("QMBMP3File::Extract(): Couldn't open file (%s) for reading.", fullfilename.latin1());
		return -1;
	}

	//figure out buffer length, we want 30 seconds of data,
	//get rate and channels from vorbis file, and
	//specify 16 bit (2 byte) samples
	//= channels*30 sec*rate*2bytes
	bufferlen = 60*channels*sampleRate;

	//create the actual buffer
	buffer = new char[bufferlen];

	//read the mp3
	QMBmad::ReadMPEG(file, buffer, bufferlen);

	//we're finished, just need to clean up
	fclose(file);

	return ret;
}

int QMBMP3File::GetMetadata(QString filename) {
    TagLib::FileRef f(filename.latin1());
    if (f.isNull()) {
     	return -1;
    }
    
    QString artist = f.tag()->artist().toCString();
    QString title = f.tag()->title().toCString();
    QString comment = f.tag()->comment().toCString();
    QString year = ""; year.setNum(f.tag()->year());
    QString album = f.tag()->album().toCString();
    QString genre = f.tag()->genre().toCString();
    int trackNo = f.tag()->track();
    
    //get the album name
    metadata.albumName = album;
    //get the song title
    metadata.trackName = title;
    //get the artist name
    metadata.artistName = artist;
    //get the track number
    metadata.trackNum = trackNo;
    
    return 0;
}

//returns:
// 0 on succes
// -1 on invalid choice
// -2 on general fileerror (a couple of possibilities here)
// -3 on success, but old file couldn't be deleted
int QMBMP3File::Output(int resultChoice, QString format, QString newdir, bool crossfs) {

	//check if they are giving a valid choice
	if (resultChoice < 0 || resultChoice >= numresults) {
		qWarning("QMBMP3File::Output(): Invalid result choice - chose: %d, avail: %d.", resultChoice, numresults);
		return -1;
	}

	//fill in metadata with metadata from selection
	this->CopySelectedMetadata(resultChoice);
	
/*	///No more  Id3lib stuff
	//first do id3 operations on the original file
	ID3_Tag filetag(fullfilename.latin1());

	//strip out all other tags
	filetag.Strip(ID3TT_ALL);
	filetag.Clear();

	//set the album name
	ID3_AddAlbum(&filetag, queryresults[resultChoice].albumName.latin1());

	//set the song title
	ID3_AddTitle(&filetag, queryresults[resultChoice].trackName.latin1());

	//set the artist name
	ID3_AddArtist(&filetag, queryresults[resultChoice].artistName.latin1());

	//set the track number
	ID3_AddTrack(&filetag, queryresults[resultChoice].trackNum.toInt());

	//update the tags
	filetag.SetPadding(false);
	filetag.Update();

	//disassociate the tagging object from the file
	filetag.Clear(); */

	//get the directory and then set the output name, replacing placeholders as we go
	QString newfilename = format;
	newfilename.replace("{ARTIST}", queryresults[resultChoice].artistName);
	newfilename.replace("{ALBUM}", queryresults[resultChoice].albumName);
	newfilename.replace("{TRACK}", queryresults[resultChoice].trackName);
	newfilename.replace("{TRACKNUM}", queryresults[resultChoice].trackNum);
	newfilename.replace("{EXT}", "mp3");
	if (newdir.isEmpty()) {
		//no newdir specified, so just use the original
		QString dir = fullfilename.left( fullfilename.findRev("/") + 1 );
		newfilename.prepend(dir);
	} else {
		if (newdir.right(1) != "/") newdir.append("/");
		newfilename.prepend(newdir);
	}

	//now we just have to move the file
	int ret = rename(fullfilename.latin1(), newfilename.latin1());
	if (ret < 0) {
		if (errno == EXDEV) {
			//couldn't rename across filesystems, we have to do a full copy instead
			if (crossfs) {
				//this is pretty efficient, but can take a long time on large files!
				std::ifstream fin(fullfilename.latin1(), std::ios_base::in | std::ios_base::binary);
				std::ofstream fout(newfilename.latin1(), std::ios_base::out | std::ios_base::binary | std::ios_base::trunc);
				if (fin.is_open() && fout.is_open())
					fout << fin.rdbuf();
				fin.close();
				fout.close();
				if (remove(fullfilename.latin1()) < 0) {
					qWarning("QMBMP3File::Output(): Couldn't remove old file (%s) after successfully writing new file.", fullfilename.latin1());
					return -3;
				}
			} else {
				qWarning("QMBMP3File::Output(): You chose not to copy across filesystems.  The metadata has been written to the file (%s), but it has not been renamed.", fullfilename.latin1());
			}
		} else {
			qWarning("QMBMP3File::Output(): Error moving file from %s to %s.", fullfilename.latin1(), newfilename.latin1());
			return -2;
		}
	}

	//if we got here we have succeeded
	return 0;
}
